/***********************************************************************
 * demo16x24.c - Arduino demo program for Holtek HT1632 LED driver chip,
 *            As implemented on the Sure Electronics DE-DP016 display board
 *            (16*24 dot matrix LED module.)
 * Nov, 2008 by Bill Westfield ("WestfW")
 *   Copyrighted and distributed under the terms of the Berkeley license
 *   (copy freely, but include this notice of original author.)
 *
 * Dec 2010, FlorinC - adapted for 3216 display
 ***********************************************************************/

#include <WProgram.h>
#include <avr/pgmspace.h>
#include "ht1632.h"
#include "font3.h" 
#include "PS2Keyboard.h"


#define Number_of_Displays 4
#define CHIP_MAX 4*Number_of_Displays //Four HT1632Cs on one board
#define X_MAX 32*Number_of_Displays -1
#define Y_MAX 15

#define CLK_DELAY


// possible values for a pixel;
//RANDOMCOLOR for each line a different color
//MULTICOLOR for each point a different random color
#define BLACK  0
#define GREEN  1
#define RED    2
#define ORANGE 3
#define RANDOMCOLOR 4 // only for scrolling functions
#define MULTICOLOR 5

// insert here the number of columns of your font files 
// the compiler will comment how large the number of columns
// should be
#define NCOLUMNS 8

#define NCOLUMNS2 8

#define LONGDELAY 1000  // This delay BETWEEN demos


#define plot(x,y,v)  ht1632_plot(x,y,v)

#define plot2(x2,y2,v2)  ht1632_plot2(x2,y2,v2)

#define cls          ht1632_clear

#define cls2         ht1632_clear2


const int DataPin = 0; //Brown wire
const int IRQpin =  1; //Orange wire

PS2Keyboard keyboard;

int PositionX = 2; //Center the display a little
int Counter = 0;


// our own copy of the "video" memory; 64 bytes for each of the 4 screen quarters;
// each 64-element array maps 2 planes:
// indexes from 0 to 31 are allocated for green plane;
// indexes from 32 to 63 are allocated for red plane;
// when a bit is 1 in both planes, it is displayed as orange (green + red);
byte ht1632_shadowram[64][4*Number_of_Displays] = {0};

byte ht1632_shadowram2[64][4*Number_of_Displays] = {0};

/*
 * Set these constants to the values of the pins connected to the SureElectronics Module
 */
static const byte ht1632_data = 6;  // Data pin (pin 7 of display connector)
static const byte ht1632_wrclk = 7; // Write clock pin (pin 5 of display connector)
static const byte ht1632_cs = 8;    // Chip Select (pin 1 of display connnector)
static const byte ht1632_clk = 9; // clock pin (pin 2 of display connector)

// Teensy++ 2.0 only pins for 2nd set of 4 displays
static const byte ht1632_data2 = 23;  // Data pin (pin 7 of 2nd display connector)
static const byte ht1632_wrclk2 = 22; // Write clock pin (pin 5 of 2nd display connector)
static const byte ht1632_cs2 = 21;    // Chip Select (pin 1 of 2nd display connnector)
static const byte ht1632_clk2 = 20; // clock pin (pin 2 of 2nd display connector)


//**************************************************************************************************
//Function Name: OutputCLK_Pulse
//Function Feature: enable CLK_74164 pin to output a clock pulse
//Input Argument: void
//Output Argument: void
//**************************************************************************************************
void OutputCLK_Pulse(void) //Output a clock pulse
{
  digitalWrite(ht1632_clk, HIGH);
  digitalWrite(ht1632_clk, LOW);
}


//**************************************************************************************************
//Function Name: OutputCLK_Pulse2
//Function Feature: enable CLK_74164 pin to output a clock pulse
//Input Argument: void
//Output Argument: void
//**************************************************************************************************
void OutputCLK_Pulse2(void) //Output a clock pulse
{  
  digitalWrite(ht1632_clk2, HIGH);
  digitalWrite(ht1632_clk2, LOW);
}


//**************************************************************************************************
//Function Name: OutputA_74164
//Function Feature: enable pin A of 74164 to output 0 or 1
//Input Argument: x: if x=1, 74164 outputs high. If x?1, 74164 outputs low.
//Output Argument: void
//**************************************************************************************************
void OutputA_74164(unsigned char x) //Input a digital level to 74164
{
  digitalWrite(ht1632_cs, (x==1 ? HIGH : LOW));
}


//**************************************************************************************************
//Function Name: OutputA_741642
//Function Feature: enable pin A of 74164 to output 0 or 1
//Input Argument: x: if x2=1, 74164 outputs high. If x2?1, 74164 outputs low.
//Output Argument: void
//**************************************************************************************************
void OutputA_741642(unsigned char x2) //Input a digital level to 74164
{  
  digitalWrite(ht1632_cs2, (x2==1 ? HIGH : LOW));
}


//**************************************************************************************************
//Function Name: ChipSelect
//Function Feature: enable HT1632C
//Input Argument: select: HT1632C to be selected
// If select=0, select none.
// If s<0, select all.
//Output Argument: void
//**************************************************************************************************
void ChipSelect(int select)
{
  unsigned char tmp = 0;
  if(select<0) //Enable all HT1632Cs
  {
    OutputA_74164(0);
    CLK_DELAY;
    for(tmp=0; tmp<CHIP_MAX; tmp++)
    {
      OutputCLK_Pulse();
    }
  }
  else if(select==0) //Disable all HT1632Cs
  {
    OutputA_74164(1);
    CLK_DELAY;
    for(tmp=0; tmp<CHIP_MAX; tmp++)
    {
      OutputCLK_Pulse();
    }
  }
  else
  {
    OutputA_74164(1);
    CLK_DELAY;
    for(tmp=0; tmp<CHIP_MAX; tmp++)
    {
      OutputCLK_Pulse();
    }
    OutputA_74164(0);
    CLK_DELAY;
    OutputCLK_Pulse();
    CLK_DELAY;
    OutputA_74164(1);
    CLK_DELAY;
    tmp = 1;
    for( ; tmp<select; tmp++)
    {
      OutputCLK_Pulse();
    }
  }
}


//**************************************************************************************************
//Function Name: ChipSelect2
//Function Feature: enable HT1632C
//Input Argument: select: HT1632C to be selected
// If select=0, select none.
// If s<0, select all.
//Output Argument: void
//**************************************************************************************************
void ChipSelect2(int select2)
{
  unsigned char tmp2 = 0;
  if(select2<0) //Enable all HT1632Cs
  {
    OutputA_741642(0);
    CLK_DELAY;
    for(tmp2=0; tmp2<CHIP_MAX; tmp2++)
    {
      OutputCLK_Pulse2();
    }
  }
  else if(select2==0) //Disable all HT1632Cs
  {
    OutputA_741642(1);
    CLK_DELAY;
    for(tmp2=0; tmp2<CHIP_MAX; tmp2++)
    {
      OutputCLK_Pulse2();
    }
  }
  else
  {
    OutputA_741642(1);
    CLK_DELAY;
    for(tmp2=0; tmp2<CHIP_MAX; tmp2++)
    {
      OutputCLK_Pulse2();
    }
    OutputA_741642(0);
    CLK_DELAY;
    OutputCLK_Pulse2();
    CLK_DELAY;
    OutputA_741642(1);
    CLK_DELAY;
    tmp2 = 1;
    for( ; tmp2<select2; tmp2++)
    {
      OutputCLK_Pulse2();
    }
  }
}


/*
 * ht1632_writebits
 * Write bits (up to 8) to h1632 on pins ht1632_data, ht1632_wrclk
 * Chip is assumed to already be chip-selected
 * Bits are shifted out from MSB to LSB, with the first bit sent
 * being (bits & firstbit), shifted till firsbit is zero.
 */
void ht1632_writebits (byte bits, byte firstbit)
{
  DEBUGPRINT(" ");
  while (firstbit) {
    DEBUGPRINT((bits&firstbit ? "1" : "0"));
    digitalWrite(ht1632_wrclk, LOW);
    if (bits & firstbit) {
      digitalWrite(ht1632_data, HIGH);
    } 
    else {
      digitalWrite(ht1632_data, LOW);
    }
    digitalWrite(ht1632_wrclk, HIGH);
    firstbit >>= 1;
  }
}


/*
 * ht1632_writebits2
 * Write bits (up to 8) to h1632 on pins ht1632_data2, ht1632_wrclk2
 * Chip is assumed to already be chip-selected
 * Bits are shifted out from MSB to LSB, with the first bit sent
 * being (bits2 & firstbit2), shifted till firsbit2 is zero.
 */
void ht1632_writebits2 (byte bits2, byte firstbit2)
{
  DEBUGPRINT(" ");
  while (firstbit2) {
    DEBUGPRINT((bits&firstbit2 ? "1" : "0"));
    digitalWrite(ht1632_wrclk2, LOW);
    if (bits2 & firstbit2) {
      digitalWrite(ht1632_data2, HIGH);
    } 
    else {
      digitalWrite(ht1632_data2, LOW);
    }
    digitalWrite(ht1632_wrclk2, HIGH);
    firstbit2 >>= 1;
  }
}


/*
 * ht1632_sendcmd
 * Send a command to the ht1632 chip.
 */
static void ht1632_sendcmd (byte chipNo, byte command)
{
  ChipSelect(chipNo);
  ht1632_writebits(HT1632_ID_CMD, 1<<2);  // send 3 bits of id: COMMMAND
  ht1632_writebits(command, 1<<7);  // send the actual command
  ht1632_writebits(0, 1); 	/* one extra dont-care bit in commands. */
  ChipSelect(0);
}

/*
 * ht1632_sendcmd2
 * Send a command to the ht1632 chip.
 */
static void ht1632_sendcmd2 (byte chipNo2, byte command2)
{
  ChipSelect2(chipNo2);
  ht1632_writebits2(HT1632_ID_CMD, 1<<2);  // send 3 bits of id: COMMMAND
  ht1632_writebits2(command2, 1<<7);  // send the actual command
  ht1632_writebits2(0, 1); 	/* one extra dont-care bit in commands. */
  ChipSelect2(0);
}


/*
 * ht1632_senddata
 * send a nibble (4 bits) of data to a particular memory location of the
 * ht1632.  The command has 3 bit ID, 7 bits of address, and 4 bits of data.
 *    Select 1 0 1 A6 A5 A4 A3 A2 A1 A0 D0 D1 D2 D3 Free
 * Note that the address is sent MSB first, while the data is sent LSB first!
 * This means that somewhere a bit reversal will have to be done to get
 * zero-based addressing of words and dots within words.
 */
static void ht1632_senddata (byte chipNo, byte address, byte data)
{
  ChipSelect(chipNo);
  ht1632_writebits(HT1632_ID_WR, 1<<2);  // send ID: WRITE to RAM
  ht1632_writebits(address, 1<<6); // Send address
  ht1632_writebits(data, 1<<3); // send 4 bits of data
  ChipSelect(0);
}


/*
 * ht1632_senddata2
 * send a nibble (4 bits) of data to a particular memory location of the
 * ht1632.  The command has 3 bit ID, 7 bits of address, and 4 bits of data.
 *    Select 1 0 1 A6 A5 A4 A3 A2 A1 A0 D0 D1 D2 D3 Free
 * Note that the address is sent MSB first, while the data is sent LSB first!
 * This means that somewhere a bit reversal will have to be done to get
 * zero-based addressing of words and dots within words.
 */
static void ht1632_senddata2 (byte chipNo2, byte address2, byte data2)
{
  ChipSelect2(chipNo2);
  ht1632_writebits2(HT1632_ID_WR, 1<<2);  // send ID: WRITE to RAM
  ht1632_writebits2(address2, 1<<6); // Send address
  ht1632_writebits2(data2, 1<<3); // send 4 bits of data
  ChipSelect2(0);
}


void ht1632_setup()
{
  pinMode(ht1632_cs, OUTPUT);
  digitalWrite(ht1632_cs, HIGH); 	/* unselect (active low) */
  pinMode(ht1632_wrclk, OUTPUT);
  pinMode(ht1632_data, OUTPUT);
  pinMode(ht1632_clk, OUTPUT);

  for (int j=1; j<=CHIP_MAX; j++)
  {
    ht1632_sendcmd(j, HT1632_CMD_SYSDIS);  // Disable system
    ht1632_sendcmd(j, HT1632_CMD_COMS00);
    ht1632_sendcmd(j, HT1632_CMD_MSTMD); 	/* Master Mode */
    ht1632_sendcmd(j, HT1632_CMD_RCCLK);  // HT1632C
    ht1632_sendcmd(j, HT1632_CMD_SYSON); 	/* System on */
    ht1632_sendcmd(j, HT1632_CMD_LEDON); 	/* LEDs on */
  
  for (byte i=0; i<96; i++)
  {
    ht1632_senddata(j, i, 0);  // clear the display!
  }
  }
}


void ht1632_setup2()
{
  pinMode(ht1632_cs2, OUTPUT);
  digitalWrite(ht1632_cs2, HIGH); 	/* unselect (active low) */
  pinMode(ht1632_wrclk2, OUTPUT);
  pinMode(ht1632_data2, OUTPUT);
  pinMode(ht1632_clk2, OUTPUT);

  for (int j2=1; j2<=CHIP_MAX; j2++)
  {
    ht1632_sendcmd2(j2, HT1632_CMD_SYSDIS);  // Disable system
    ht1632_sendcmd2(j2, HT1632_CMD_COMS00);
    ht1632_sendcmd2(j2, HT1632_CMD_MSTMD); 	/* Master Mode */
    ht1632_sendcmd2(j2, HT1632_CMD_RCCLK);  // HT1632C
    ht1632_sendcmd2(j2, HT1632_CMD_SYSON); 	/* System on */
    ht1632_sendcmd2(j2, HT1632_CMD_LEDON); 	/* LEDs on */
  
  for (byte i2=0; i2<96; i2++)
  {
    ht1632_senddata2(j2, i2, 0);  // clear the display!
  }
  }
}


//**************************************************************************************************
//Function Name: xyToIndex
//Function Feature: get the value of x,y
//Input Argument: x: X coordinate
//                y: Y coordinate
//Output Argument: address of xy
//**************************************************************************************************
byte xyToIndex(byte x, byte y)
{
  byte nChip, addr;

//if (x>=32) {
//  nChip = 3 + x/16 + (y>7?2:0);
//} else {
//  nChip = 1 + x/16 + (y>7?2:0);
//} 
  
  if (x>=96) {
    nChip = 7 + x/16 + (y>7?2:0);
  } 
  else if (x>=64) {
    nChip = 5 + x/16 + (y>7?2:0);
  }    
   else if (x>=32) {
    nChip = 3 + x/16 + (y>7?2:0);
  }  
  else {
    nChip = 1 + x/16 + (y>7?2:0);
  } 

  x = x % 16;
  y = y % 8;
  addr = (x<<1) + (y>>2);

  return addr;
}

//**************************************************************************************************
//Function Name: xyToIndex2
//Function Feature: get the value of x2,y2
//Input Argument: x2: X2 coordinate
//                y2: Y2 coordinate
//Output Argument: address of x2y2
//**************************************************************************************************
byte xyToIndex2(byte x2, byte y2)
{
  byte nChip2, addr2;

//if (x>=32) {
//  nChip = 3 + x/16 + (y>7?2:0);
//} else {
//  nChip = 1 + x/16 + (y>7?2:0);
//} 
  
  if (x2>=96) {
    nChip2 = 7 + x2/16 + (y2>7?2:0);
  } 
  else if (x2>=64) {
    nChip2 = 5 + x2/16 + (y2>7?2:0);
  }    
   else if (x2>=32) {
    nChip2 = 3 + x2/16 + (y2>7?2:0);
  }  
  else {
    nChip2 = 1 + x2/16 + (y2>7?2:0);
  } 

  x2 = x2 % 16;
  y2 = y2 % 8;
  addr2 = (x2<<1) + (y2>>2);

  return addr2;
}

//**************************************************************************************************
//Function Name: calcBit
//Function Feature: calculate the bitval of y
//Input Argument: y: Y coordinate
//Output Argument: bitval
//**************************************************************************************************
#define calcBit(y) (8>>(y&3))

//**************************************************************************************************
//Function Name: calcBit2
//Function Feature: calculate the bitval of y2
//Input Argument: y2: Y2 coordinate
//Output Argument: bitval2
//**************************************************************************************************
#define calcBit2(y2) (8>>(y2&3))


//**************************************************************************************************
//Function Name: get_pixel
//Function Feature: get the value of x,y
//Input Argument: x: X coordinate
//                y: Y coordinate
//Output Argument: color setted on x,y coordinates
//**************************************************************************************************
int get_pixel(byte x, byte y) {
  byte addr, bitval, nChip;

//if (x>=32) {
//  nChip = 3 + x/16 + (y>7?2:0);
//} else {
//  nChip = 1 + x/16 + (y>7?2:0);
//} 
  
  if (x>=96) {
    nChip = 7 + x/16 + (y>7?2:0);
  } 
  else if (x>=64) {
    nChip = 5 + x/16 + (y>7?2:0);
  }    
   else if (x>=32) {
    nChip = 3 + x/16 + (y>7?2:0);
  }  
  else {
    nChip = 1 + x/16 + (y>7?2:0);
  } 

  addr = xyToIndex(x,y);
  bitval = calcBit(y);

  if ((ht1632_shadowram[addr][nChip-1] & bitval) && (ht1632_shadowram[addr+32][nChip-1] & bitval)) {
    return ORANGE;
  } else if (ht1632_shadowram[addr][nChip-1] & bitval) {
    return GREEN;
  } else if (ht1632_shadowram[addr+32][nChip-1] & bitval) {
    return RED;
  } else {
    return 0;
  }
}


//**************************************************************************************************
//Function Name: get_pixel2
//Function Feature: get the value of x2,y2
//Input Argument: x2: X2 coordinate
//                y2: Y2 coordinate
//Output Argument: color setted on x2,y2 coordinates
//**************************************************************************************************
int get_pixel2(byte x2, byte y2) {
  byte addr2, bitval2, nChip2;

//if (x>=32) {
//  nChip = 3 + x/16 + (y>7?2:0);
//} else {
//  nChip = 1 + x/16 + (y>7?2:0);
//} 
  
  if (x2>=96) {
    nChip2 = 7 + x2/16 + (y2>7?2:0);
  } 
  else if (x2>=64) {
    nChip2 = 5 + x2/16 + (y2>7?2:0);
  }    
   else if (x2>=32) {
    nChip2 = 3 + x2/16 + (y2>7?2:0);
  }  
  else {
    nChip2 = 1 + x2/16 + (y2>7?2:0);
  } 

  addr2 = xyToIndex2(x2,y2);
  bitval2 = calcBit2(y2);

  if ((ht1632_shadowram2[addr2][nChip2-1] & bitval2) && (ht1632_shadowram2[addr2+32][nChip2-1] & bitval2)) {
    return ORANGE;
  } else if (ht1632_shadowram2[addr2][nChip2-1] & bitval2) {
    return GREEN;
  } else if (ht1632_shadowram2[addr2+32][nChip2-1] & bitval2) {
    return RED;
  } else {
    return 0;
  }
}



/*
 * plot a point on the display, with the upper left hand corner
 * being (0,0), and the lower right hand corner being (31, 15);
 * parameter "color" could have one of the 4 values:
 * black (off), red, green or yellow;
 */
//void ht1632_plot (byte x, byte y, byte color)
void ht1632_plot (int x, int y, byte color)
{
   byte nChip, addr, bitval;
  
  if (x<0 || x>X_MAX || y<0 || y>Y_MAX)
    return;
  
  if (color != BLACK && color != GREEN && color != RED && color != ORANGE)
    return;
  
//if (x>=32) {
//  nChip = 3 + x/16 + (y>7?2:0);
//} else {
//  nChip = 1 + x/16 + (y>7?2:0);
//} 
  
  if (x>=96) {
    nChip = 7 + x/16 + (y>7?2:0);
  } 
  else if (x>=64) {
    nChip = 5 + x/16 + (y>7?2:0);
  }    
   else if (x>=32) {
    nChip = 3 + x/16 + (y>7?2:0);
  }  
  else {
    nChip = 1 + x/16 + (y>7?2:0);
  } 
  
  addr = xyToIndex(x,y);
  bitval = calcBit(y);
  
  switch (color)
  {
    case BLACK:
      if (get_pixel(x,y) != BLACK) { // compare with memory to only set if pixel is other color
        // clear the bit in both planes;
        ht1632_shadowram[addr][nChip-1] &= ~bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
        addr = addr + 32;
        ht1632_shadowram[addr][nChip-1] &= ~bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
      }
      break;
    case GREEN:
      if (get_pixel(x,y) != GREEN) { // compare with memory to only set if pixel is other color
        // set the bit in the green plane and clear the bit in the red plane;
        ht1632_shadowram[addr][nChip-1] |= bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
        addr = addr + 32;
        ht1632_shadowram[addr][nChip-1] &= ~bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
      }
      break;
    case RED:
      if (get_pixel(x,y) != RED) { // compare with memory to only set if pixel is other color
        // clear the bit in green plane and set the bit in the red plane;
        ht1632_shadowram[addr][nChip-1] &= ~bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
        addr = addr + 32;
        ht1632_shadowram[addr][nChip-1] |= bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
      }
      break;
    case ORANGE:
      if (get_pixel(x,y) != ORANGE) { // compare with memory to only set if pixel is other color
        // set the bit in both the green and red planes;
        ht1632_shadowram[addr][nChip-1] |= bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
        addr = addr + 32;
        ht1632_shadowram[addr][nChip-1] |= bitval;
        ht1632_senddata(nChip, addr, ht1632_shadowram[addr][nChip-1]);
      }
      break;
  }
}


/*
 * plot a point on the display, with the upper left hand corner
 * being (0,0), and the lower right hand corner being (31, 15);
 * parameter "color" could have one of the 4 values:
 * black (off), red, green or yellow;
 */
//void ht1632_plot2 (byte x, byte y, byte color)
void ht1632_plot2 (int x2, int y2, byte color2)
{
   byte nChip2, addr2, bitval2;
  
  if (x2<0 || x2>X_MAX || y2<0 || y2>Y_MAX)
    return;
  
  if (color2 != BLACK && color2 != GREEN && color2 != RED && color2 != ORANGE)
    return;
  
//if (x>=32) {
//  nChip = 3 + x/16 + (y>7?2:0);
//} else {
//  nChip = 1 + x/16 + (y>7?2:0);
//} 
  
  if (x2>=96) {
    nChip2 = 7 + x2/16 + (y2>7?2:0);
  } 
  else if (x2>=64) {
    nChip2 = 5 + x2/16 + (y2>7?2:0);
  }    
   else if (x2>=32) {
    nChip2 = 3 + x2/16 + (y2>7?2:0);
  }  
  else {
    nChip2 = 1 + x2/16 + (y2>7?2:0);
  } 
  
  addr2 = xyToIndex2(x2,y2);
  bitval2 = calcBit2(y2);
  
  switch (color2)
  {
    case BLACK:
      if (get_pixel2(x2,y2) != BLACK) { // compare with memory to only set if pixel is other color
        // clear the bit in both planes;
        ht1632_shadowram2[addr2][nChip2-1] &= ~bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
        addr2 = addr2 + 32;
        ht1632_shadowram2[addr2][nChip2-1] &= ~bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
      }
      break;
    case GREEN:
      if (get_pixel2(x2,y2) != GREEN) { // compare with memory to only set if pixel is other color
        // set the bit in the green plane and clear the bit in the red plane;
        ht1632_shadowram2[addr2][nChip2-1] |= bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
        addr2 = addr2 + 32;
        ht1632_shadowram2[addr2][nChip2-1] &= ~bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
      }
      break;
    case RED:
      if (get_pixel2(x2,y2) != RED) { // compare with memory to only set if pixel is other color
        // clear the bit in green plane and set the bit in the red plane;
        ht1632_shadowram2[addr2][nChip2-1] &= ~bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
        addr2 = addr2 + 32;
        ht1632_shadowram2[addr2][nChip2-1] |= bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
      }
      break;
    case ORANGE:
      if (get_pixel2(x2,y2) != ORANGE) { // compare with memory to only set if pixel is other color
        // set the bit in both the green and red planes;
        ht1632_shadowram2[addr2][nChip2-1] |= bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
        addr2 = addr2 + 32;
        ht1632_shadowram2[addr2][nChip2-1] |= bitval2;
        ht1632_senddata2(nChip2, addr2, ht1632_shadowram2[addr2][nChip2-1]);
      }
      break;
  }
}


/*
 * ht1632_clear
 * clear the display, and the shadow memory, and the snapshot
 * memory.  This uses the "write multiple words" capability of
 * the chipset by writing all 96 words of memory without raising
 * the chipselect signal.
 */
void ht1632_clear()
{
  char i;
  //for (int i=1; i<=CHIP_MAX; i++)
  for (int i=1; i<=8; i++)
  {
    ChipSelect(-1);
    ht1632_writebits(HT1632_ID_WR, 1<<2);  // send ID: WRITE to RAM
    ht1632_writebits(0, 1<<6); // Send address
    for (i = 0; i < 96/2; i++) // Clear entire display
      ht1632_writebits(0, 1<<7); // send 8 bits of data
    ChipSelect(0);

    for (int j=0; j < 64; j++)
      ht1632_shadowram[j][i] = 0;
  }
}


/*
 * ht1632_clear2
 * clear the display, and the shadow memory, and the snapshot
 * memory.  This uses the "write multiple words" capability of
 * the chipset by writing all 96 words of memory without raising
 * the chipselect signal.
 */
void ht1632_clear2()
{
  char i2;
  //for (int i2=1; i2<=CHIP_MAX; i2++)
  for (int i2=1; i2<=8; i2++)
  {
    ChipSelect2(-1);
    ht1632_writebits2(HT1632_ID_WR, 1<<2);  // send ID: WRITE to RAM
    ht1632_writebits2(0, 1<<6); // Send address
    for (i2 = 0; i2 < 96/2; i2++) // Clear entire display
      ht1632_writebits2(0, 1<<7); // send 8 bits of data
    ChipSelect2(0);

    for (int j2=0; j2 < 64; j2++)
      ht1632_shadowram2[j2][i2] = 0;
  }
}


//void ht1632_putchar(byte x, byte y, char c, byte color=GREEN)
void ht1632_putchar(int x, int y, char c, byte color=GREEN)
{
  byte dots;
  //if (c >= 'A' && c <= 'Z' ||
  //  (c >= 'a' && c <= 'z') ) {
  //  c &= 0x1F;   // A-Z maps to 1-26
  //} 
  //else if (c >= '0' && c <= '9') {
  //  c = (c - '0') + 27;
  //} 
  //else if (c == ' ') {
  //  c = 0; // space
  //}
  
  if (c == ' ') {c = 0;}
  else if (c == '!') {c = 1;}
  else if (c == '"') {c = 2;}
  else if (c == '#') {c = 3;}
  else if (c == '$') {c = 4;}
  else if (c == '%') {c = 5;}
  else if (c == '&') {c = 6;}
  //else if (c == ''') {c = 7;}
  else if (c == '(') {c = 8;}
  else if (c == ')') {c = 9;}
  else if (c == '*') {c = 10;}
  else if (c == '+') {c = 11;}
  else if (c == ',') {c = 12;}
  else if (c == '-') {c = 13;}
  else if (c == '.') {c = 14;}
  else if (c == '/') {c = 15;}  
  
  else if (c >= '0' && c <= '9') {
    c = (c - '0') + 16;
  }
  
  else if (c == ':') {c = 26;} 
  else if (c == ';') {c = 27;} 
  else if (c == '<') {c = 28;}
  else if (c == '=') {c = 29;}
  else if (c == '>') {c = 30;}
  else if (c == '?') {c = 31;}
  else if (c == '@') {c = 32;}   
  
  else if (c >= 'A' && c <= 'Z') {
    c = (c - 'A') + 33;
  }
  
  else if (c == '[') {c = 59;}
  //else if (c == '\') {c = 60;}
  else if (c == ']') {c = 61;}
  else if (c == '^') {c = 62;}
  else if (c == '_') {c = 63;}
  else if (c == '`') {c = 64;}
  
  else if (c >= 'a' && c <= 'z') {
    c = (c - 'a') + 65;
  }
  
  else if (c == '{') {c = 91;}
  else if (c == '|') {c = 92;}
  else if (c == '}') {c = 93;}
  
  for (char col=0; col< 6; col++) {
    dots = pgm_read_byte_near(&my3font[c][col]);
    for (char row=0; row <=7; row++) {
      if (dots & (64>>row))   	     // only 7 rows.
        plot(x+col, y+row, color);
      else 
        plot(x+col, y+row, 0);
    }
  }
}


//void ht1632_putchar2(byte x2, byte y2, char c2, byte color2=GREEN)
void ht1632_putchar2(int x2, int y2, char c2, byte color2=GREEN)
{
  byte dots2;
  //if (c >= 'A' && c <= 'Z' ||
  //  (c >= 'a' && c <= 'z') ) {
  //  c &= 0x1F;   // A-Z maps to 1-26
  //} 
  //else if (c >= '0' && c <= '9') {
  //  c = (c - '0') + 27;
  //} 
  //else if (c == ' ') {
  //  c = 0; // space
  //}
  
  if (c2 == ' ') {c2 = 0;}
  else if (c2 == '!') {c2 = 1;}
  else if (c2 == '"') {c2 = 2;}
  else if (c2 == '#') {c2 = 3;}
  else if (c2 == '$') {c2 = 4;}
  else if (c2 == '%') {c2 = 5;}
  else if (c2 == '&') {c2 = 6;}
  //else if (c == ''') {c = 7;}
  else if (c2 == '(') {c2 = 8;}
  else if (c2 == ')') {c2 = 9;}
  else if (c2 == '*') {c2 = 10;}
  else if (c2 == '+') {c2 = 11;}
  else if (c2 == ',') {c2 = 12;}
  else if (c2 == '-') {c2 = 13;}
  else if (c2 == '.') {c2 = 14;}
  else if (c2 == '/') {c2 = 15;}  
  
  else if (c2 >= '0' && c2 <= '9') {
    c2 = (c2 - '0') + 16;
  }
  
  else if (c2 == ':') {c2 = 26;} 
  else if (c2 == ';') {c2 = 27;} 
  else if (c2 == '<') {c2 = 28;}
  else if (c2 == '=') {c2 = 29;}
  else if (c2 == '>') {c2 = 30;}
  else if (c2 == '?') {c2 = 31;}
  else if (c2 == '@') {c2 = 32;}   
  
  else if (c2 >= 'A' && c2 <= 'Z') {
    c2 = (c2 - 'A') + 33;
  }
  
  else if (c2 == '[') {c2 = 59;}
  //else if (c2 == '\') {c2 = 60;}
  else if (c2 == ']') {c2 = 61;}
  else if (c2 == '^') {c2 = 62;}
  else if (c2 == '_') {c2 = 63;}
  else if (c2 == '`') {c2 = 64;}
  
  else if (c2 >= 'a' && c2 <= 'z') {
    c2 = (c2 - 'a') + 65;
  }
  
  else if (c2 == '{') {c2 = 91;}
  else if (c2 == '|') {c2 = 92;}
  else if (c2 == '}') {c2 = 93;}
  
  for (char col2=0; col2< 6; col2++) {
    dots2 = pgm_read_byte_near(&my3font[c2][col2]);
    for (char row2=0; row2 <=7; row2++) {
      if (dots2 & (64>>row2))   	     // only 7 rows.
        plot2(x2+col2, y2+row2, color2);
      else 
        plot2(x2+col2, y2+row2, 0);
    }
  }
}


/***********************************************************************
 * traditional Arduino sketch functions: setup and loop.
 ***********************************************************************/

void setup ()  // flow chart from page 17 of datasheet
{
  ht1632_setup(); 
  ht1632_setup2(); 
  
  keyboard.begin(DataPin, IRQpin);

  cls();
  cls2();
  
}


void loop ()
{
    
if (keyboard.available()) {

  // read the next key
    char c = keyboard.read();

    if (Counter >= 0 && Counter <= 20) {
      ht1632_putchar(PositionX, 0, c, GREEN);
      Counter++;
      PositionX = PositionX + 6;
    }
     
    if (Counter > 20 && Counter <= 42) {
      ht1632_putchar(PositionX, 8, c, RED);
      Counter++;
      PositionX = PositionX + 6;
      }
    
    if (Counter > 42 && Counter <= 64) {
      ht1632_putchar2(PositionX, 0, c, ORANGE);
      Counter++;
      PositionX = PositionX + 6;
      }
      
    if (Counter > 64 && Counter <= 86) {
      ht1632_putchar2(PositionX, 8, c, GREEN);
      Counter++;
      PositionX = PositionX + 6;
      }
    
     if (PositionX > 122) {
        PositionX = 2;
      } 
     
     if (Counter > 86) {
        Counter = 0;
        cls(); 
        cls2(); 
      }  
    
}
 
}




